(*  :Title:	Continuous-Time Fourier Transform  *)

(*
    :Authors:	Brian L. Evans and Joseph M. Winograd with
		Robert H. Bamberger, Wallace B. McClure, and James H. McClellan
 *)

(*  
    :Summary:	This package provides forward and inverse multidimensional
		continuous Fourier transforms defined the usual EE way
		(i.e., not normalized).
 *)

(*  :Context:	SignalProcessing`Analog`Fourier`  *)

(*  :PackageVersion:  SPP 2.9  @(#)Fourier.m	1.1	2/8/94  *)

(*
    :Copyright:	Copyright 1989-1994 by Brian L. Evans
		Georgia Tech Research Corporation
		All Rights Reserved

	Permission to use, copy, and distribute this software and
	its documentation is hereby granted, provided that no profit
	is made, that both the copyright notice and this permission
	notice appear in all copies, that both that copyright notice
	and this permission notice appear in supporting documentation,
	and that the name of the Georgia Tech Research Corporation
	(hereafter "GTRC"), Georgia Tech, or Georgia Institute of
	Technology not be used in advertising or publicity pertaining
	to distribution of the software without specific, written prior
	permission by the GTRC.  Permission to modify this software for
	personal use is also hereby granted provided that the modified
	version of this software is neither copied nor distributed.
	Copying or distributing modified versions of this software,
	making derivative works of this software, and marketing this
	software for profit are prohibited except with specific, written
	permission by the GTRC or by any entity to which the GTRC grants
	this authority.  GTRC makes no representations about the
	suitability of this software for any purpose.  It is provided
	"as is" without express or implied warranty.
 *)

(*
    :History:	start -- February 22, 1990
		made into package -- April 18, 1990
		added Dialogue ability -- October, 1990
		added many more transform pairs -- February, 1991
		incorporated duality so only one set of transform
		  pairs, properties, etc., is encoded --  December, 1991
		robust for piecewise polynomials -- January, 1993 (JMW)
		fixes related to sampling theory -- January, 1993 (JMW)
		restructure knowledge base for true mD -- May, 1993 (JMW)
		fix for the integration rule -- February 1, 1994 (RHB)
 *)

(*  :Keywords:	Fourier transform, frequency response  *)

(*
    :Source:	{Signals and Systems} by A. V. Oppenheim and A. S. Willsky.
		  Prentice Hall.
		{The Fourier Transform and Its Applications} by R. Bracewell
 *)

(*  :Warning:	*)

(*  :Mathematica Version:  1.2 or 2.0  *)

(*
    :Limitation:   If the function to  be transformed does not match either
		   a specific Fourier transform or the LaPlace transform, a 
		   nasty expression is returned.
 *)

(*
    :Discussion: When loaded into Mathematica,  this package will allow the 
		 computation of  continuous  Fourier  and  inverse  Fourier
		 transforms.   It  turns  out  that in order to show a user
		 each step  of  transformation process, the transform rules
		 needed to be placed in a list of deferred (delayed) rules.

	MyFourier[f, t, w, s] computes the  multidimensional Fourier trans-
	form  of  f  using  t  as  the time variable and w as the frequency
	variable.

	MyInverseCTFT[f, w, t, s] computes  the   inverse  multidimensional
	continuous-time Fourier transform using w as the frequency variable
	and t as the time variable.

	The Fourier  transform  formulas  used  follow Oppenheim & Willsky.
	To convert Bracewell's formulas,  which use 2 Pi s x instead of w t
	in the exponential function,

	  (1) use  x  = t  and  s = w / (2 Pi)  OR
	  (2) use  s = w  and  x = t / (2 Pi)  and  divide the time
	      function by 2 Pi.

	Note that in (2), dividing the time function by 2 Pi is equivalent
	to multiplying the frequency response by 2 Pi.   Either way places
	the 1/(2 Pi) normalization term in front of the inverse formula.
 *)

(*  :Functions:	CTFTransform InvCTFTransform  *)



(*  B E G I N     P A C K A G E  *)

BeginPackage[ "SignalProcessing`Analog`Fourier`",
	      "SignalProcessing`Analog`LaPlace`",
	      "SignalProcessing`Analog`InvLaPlace`",
	      "SignalProcessing`Analog`LSupport`",
	      "SignalProcessing`Digital`DSupport`",
	      "SignalProcessing`Support`TransSupport`",
	      "SignalProcessing`Support`ROC`",
	      "SignalProcessing`Support`Convolution`",
	      "SignalProcessing`Support`SigProc`",
	      "SignalProcessing`Support`SupCode`",
	      "SignalProcessing`Support`Lisp`" ]


If [ TrueQ[ $VersionNumber >= 2.0 ],
     Off[ General::spell ];
     Off[ General::spell1 ] ];


(*  U S A G E     I N F O R M A T I O N  *)

CTFTData::usage =
	"CTFTData is the data-tag for the CTFT."

CTFTransform::usage =
	"CTFTransform[f, t] or CTFTransform[f, t, w] computes the \
	continuous-time Fourier transform of f. \
	 The time variable is t, and the frequency variable is w. \
	 In multiple dimensions, t and w become lists of time and \
	frequency variables, respectively."

InvCTFTransform::usage =
	"InvCTFTransform[f, w, t] computes the inverse continuous time \
	Fourier transform of f with frequency variable w and time \
	variable t. \
	 In multiple dimensions, w and t become lists of frequency and \
	time variables, respectively."

(*  E N D     U S A G E     I N F O R M A T I O N  *)


Begin[ "`Private`" ]


(*  Messages  *)

Transform::laplace = 
	"The Region of Convergence in the Laplace transform `` does not contain the imaginary axis so it cannot be converted into a Fourier transform."


(*  Options for the forward and inverse Fourier transforms  *)

displayFixUp[ op_ ] :=
	Append[ fixUp[op], TransformLookup -> Replace[TransformLookup, op] ]

CTFTransform/: Options[ CTFTransform ] :=
	displayFixUp[ Join[ { Definition -> False, Dialogue -> False },
			    Options[ LaPlace ] ] ]

InvCTFTransform/: Options[ InvCTFTransform ] :=
	displayFixUp[ Join[ { Definition -> False,
			      Dialogue -> False,
			      Terms -> False },
			    Options[ InvLaPlace ] ] ]


(*  Printed forms of local objects so that Dialogue -> All works neatly  *)

replaceWith[ old_Symbol, new_ ] := old -> new
replaceWith[ {old_Symbol}, {new_} ] := old -> new
replaceWith[ old_List, new_ ] := ReplaceWith[old, new]

Format[ dualFT[ e_, w_, t_, rest__, InvCTFTransform ] ] :=
	SequenceForm[ ColumnForm[ { "F" Superscript[-1],
				    "  " ~StringJoin~
				    ToString[ replaceWith[w, t] ] } ],
		      { e } ]

Format[ dualFT[ e_, t_, w_, rest__, CTFTransform ] ] :=
	SequenceForm[
	  ColumnForm[ { "F",
			SequenceForm[ "  ", replaceWith[t, w] ] } ], { e } ]

Format[ derivative[e_, w_Symbol, k_] ] := 
	SequenceForm[ ColumnForm[{"D" Superscript[k], SequenceForm["  ", w]}],
		      { e } ]

Format[ integrate[e_, limits_] ] := 
	SequenceForm[ "Integrate[", e, ", ", limits, "]" ]

Format[	substitute[e_, w_, wnew1_, Plus, wnew2_] ] :=
	SequenceForm[ {e}, Subscript[ replaceWith[w, wnew1] ],
		      " + ", {e}, Subscript[ replaceWith[w, wnew2] ] ]

Format[	substitute[e_, w_, wnew1_, Minus, wnew2_] ] :=
	SequenceForm[ {e}, Subscript[ replaceWith[w, wnew1] ],
		      " - ", {e}, Subscript[ replaceWith[w, wnew2] ] ]

Format[	substitute[e_, w_, wnew_] ] :=
	SequenceForm[ {e}, Subscript[ replaceWith[w, wnew] ] ]

Format[	CTFTtoL[e_, w_, s_] ] :=
	SequenceForm[ {e}, Subscript[ replaceWith[w, s / I] ] ]

Format[ SignalProcessing`Analog`Fourier`Private`s ] := "s"
Format[ SignalProcessing`Analog`Fourier`Private`u ] := "u"


(*  State definition for the unified rule base *)

absfield = 1		(* check for Abs[t] forms			*)
expandfield = 2		(* apply expand to the expression		*)
forlaplacefield = 3	(* all else has failed so try forward Laplace	*)
invlaplacefield = 4	(* all else has failed so try inverse Laplace	*)
definitionfield = 5	(* apply the Fourier transform definition	*)
integratepulsefield = 6	(* attempt to integrate a pulse window (JMW)	*)

statevariables = 6

(*  These are not semaphore fields but lists of variables tried for	*)
(*  forward and inverse Laplace transforms applied to current expr.	*)
formultivaluedfield = 7
invmultivaluedfield = 8

(*  Global variables  *)
origVars = {}
tranVars = {}

(*  Supporting routines  *)

applyForwardLaplace[f_, t_, w_, s_, oplist_, state_, dir_] :=
	Block [	{forwardlaplacelist, ltrans, newstate, numdims,
		 svars, tvars, wvars},
		numdims = If [ AtomQ[t], 1, Length[t] ];
		{tvars, svars, wvars} =
			If [ Length[t] == 1,
			     { First[t], First[s], First[w] },
			     { t, s, w } ];
		ltrans = LaPlace[ f, tvars, svars,
				  { Dialogue -> False } ~Join~ oplist ];
		If [ InformUserQ[oplist],
		     Print[ "( after taking the Laplace transform of " ];
		     Print[ "  ", f ];
		     Print[ "  with respect to ", tvars, " and" ];
		     Print[ "  adjusting the result )" ] ];
		forwardlaplacelist = GetStateField[state, formultivaluedfield];
		newstate = SetStateField[ state, formultivaluedfield,
					  Append[forwardlaplacelist, t] ];
		If [ SameQ[Head[ltrans], LTransData],
		     If [ ROCincludesImagAxis[ ltrans, oplist ],
			  ( TheFunction[ltrans] /. ReplaceWith[s, I w] ),
			  dualFT[ f, t, w, s, oplist, newstate, dir ] ],
		     MapAll[ LtoCTFT[#1, t, w, s, oplist, newstate, dir]&,
			     ltrans ] ] ]

applyInverseLaplace[x_, w_, t_, s_, oplist_, state_, dir_] :=
	Block [	{inverselaplacelist, newstate, newx, numdims, rules,
		 sstrings, svars, tvars},
		numdims = If [ AtomQ[w], 1, Length[w] ];
		scale = (2 Pi)^numdims;
		newx = scale ( x /. ReplaceWith[w, (s / I)] );
		{tvars, svars} =
		  If [ Length[t] == 1, { First[t], First[s] }, { t, s } ];
		invltrans = InvLaPlace[ newx, svars, tvars,
					{ Dialogue -> False } ~Join~ oplist ];
		If [ InformUserQ[oplist],
		     sstrings = Map[ StripPackage, ToList[s] ];
		     rules = ReplaceWith[s, sstrings];
		     Print[ "( after taking the inverse Laplace transform of" ];
		     Print[ "  ", newx /. rules ];
		     Print[ "  with respect to ", svars /. rules,
			    " which yields" ];
		     Print[ "  ", invltrans /. rules ];
		     Print[ "  and adjusting the result )" ] ];
		inverselaplacelist = GetStateField[state, invmultivaluedfield];
		newstate = SetStateField[ state, invmultivaluedfield,
					  Append[inverselaplacelist, w] ];
		If [ MyFreeQ[invltrans, s],
		     substitute[invltrans, t, -t],   (* FT uses L^-1 so t=-t *)
		     dualFT[ x, w, t, s, oplist, newstate, dir ] ] ]

containsAbsQ[ g_, varlist_ ] :=
	Block [	{abslist = {}, flag = False},
		g /. ( Abs[v_Symbol] :> Block [ {}, flag = True; v ] /;
			 MemberQ[varlist, v] );
		flag ]

explain[ dualFT[x_, t_, rest__, CTFTransform]] :=
	Message[ Transform::incomplete, "forward Fourier transform", x, t ]
explain[ dualFT[x_, w_, rest__, InvCTFTransform]] :=
	Message[ Transform::incomplete, "inverse Fourier transform", x, w ]

fixUp[ op_ ] := { Apart -> Replace[Apart, op],
		  Definition -> Replace[Definition, op],
		  Dialogue -> Replace[Dialogue, op],
		  Simplify -> Replace[Simplify, op],
		  Terms -> Replace[Terms, op] }

hasValueAtZero[x_, t_] :=
	Block [	{limit, notvalid},
		If [ TrueQ[ $VersionNumber >= 2.0 ],
		     Off[ Infinity::indet ], Off[ Infinity::indt ] ];
		limit = Limit[x, t -> 0];
		If [ TrueQ[ $VersionNumber >= 2.0 ],
		     On[ Infinity::indet ], On[ Infinity::indt ] ];
		notvalid = SameQ[limit, Indeterminate] || InfinityQ[limit];
		! notvalid ]

heldFunctionQ[ h_ ] :=
Block [	{attributes},
	attributes = Attributes[h];
	MemberQ[attributes, HoldAll] ||
	MemberQ[attributes, HoldFirst] ||
	MemberQ[attributes, HoldRest] ]

(* initstate--- the last two fields are not semaphores but lists of *)
(*   forward and inverse Laplace variables tried (initially empty)  *)
initstate[] := Join[ Table[True, {statevariables}], {{}}, {{}} ]

(* LtoCTFT--- LaPlace[] returns the head used by its rule base *) 
LtoCTFT[ LaPlace[][x_, lrest__], t_, w_, s_, rest__ ] :=
	CTFTtoL[ dualFT[x, t, w, s, rest], w, s ]
LtoCTFT[ x_, t_, w_, s_, trest__ ] := x

makeRewriteRules[ x_Symbol ] :=
	{ ( a_ (x + b_) :>  a x + a b /; FreeQ[b, x] ),
	  ( a_ x + b_. x :> (a + b) x /; FreeQ[{a, b}, x] ) }
makeRewriteRules[ xlist_List ] :=
	{ ( a_ (x_ + b_) :>  a x + a b /;
		MemberQ[xlist, x] && MyFreeQ[b, xlist] ),
	  ( a_ x_ + b_. x :> (a + b) x /; MyFreeQ[{a, b}, xlist] ) }

mustbenegative[x_] := ! TrueQ[ Positive[x] ]
mustbepositive[x_] := ! TrueQ[ Negative[x] ]

posttransform[x_] := ( x /. postrules )
postrules = {
	CConvolve[w_, options___][f_, g_] :>
		ConvertFromList[
		  CTPiecewiseConvolution[ posttransform[f], posttransform[g],
					  w, options ],
		  w ],
	derivative[e_, t_, k_Integer] :>
		D[ posttransform[e], {t, k} ],
	integrate[e_, limits_] :>
		Integrate[ Expand[posttransform[e]], limits ],
	substitute[e_, t_Symbol, tnew1_, fun_, tnew2_] :>
		Block [	{trans},
			trans = posttransform[e];
			( trans /. t -> tnew1 ) + fun[ trans /. t -> tnew2 ] ],
	substitute[e_, t_List, tnew1_, fun_, tnew2_] :>
		Block [ {trans, i},
			trans = posttransform[e];
			For [ i = 1, i <= Length[t], i++,
			    trans = (trans /. t[[i]] -> tnew1[[i]]) + 
				fun[ trans /. t[[i]] -> tnew2[[i]] ] ] 
			trans ],
	substitute[e_, t_Symbol, tnew_] :>
		( posttransform[e] /. t -> tnew ),
	substitute[e_, t_List, tnew_] :>
		Block [ {trans, i},
		    trans = posttransform[e];
		    For [ i = 1, i <= Length[t], i++,
			trans = trans /. t[[i]] -> tnew[[i]] ];
		    trans ],
	CTFTtoL[e_] :>
		Transform[ e, -Infinity, Infinity ],
	CTFTtoL[e_, w_, s_] :>
		Transform[ substitute[e, w, s / I], -Infinity, Infinity ],
	(a_ + b_. Sign[e_]) :> 2 a CStep[-e] /; ( a == -b ),
	(a_ + a_. Sign[e_]) :> 2 a CStep[e]
}

rationalPolynomialQ[p_, t_] :=
	First[ rationalPolynomialCheck[p, t] ]
rationalPolynomialVar[p_, t_] :=
	Second[ rationalPolynomialCheck[p, t] ]
rationalPolynomialCheck[p_, t_] :=
	Block [	{flag = False, i, len, var},
		len = Length[t];
		For [ i = 1, i <= len, i++,
		      var = t[[i]];
		      If [ RationalPolynomialQ[p, var],
			   flag = True; Break[] ] ];
		{flag, var} ]

(* resolve--- key in helping decompose m-D transforms JMW *)
resolve[ rvars_, f_, t_, w_, s_, rest__ ] :=
	Block [ {r},
		r = ToList[ rvars ];
		dualFT[ f, Complement[ t, r ],
			Complement[ w, ToList[ AssociateItem[ r, t, w ] ] ],
			Complement[ s, ToList[ AssociateItem[ r, t, s ] ] ],
			rest ] ]

ROCincludesImagAxis[ trans_, op_ ] :=
	Block [	{cond, rm, rp, zerocond},
		rm = GetRMinus[trans];
		rp = GetRPlus[trans];
		cond = ( Negative[rm] && Positive[rp] );
		zerocond = ( ZeroQ[rm] || ZeroQ[rp] );
		If [ zerocond && SameQ[ Replace[Dialogue, op], All ],
		     Print[ "assuming that the origin which is an endpoint" ];
		     Print[ "of the region of convergence (ROC) of the" ];
		     Print[ "Laplace transform is included in the ROC" ] ];
		If [ cond || zerocond,
		     True,
		     False,
		     Assuming[cond, op]; True ] ]

validvarQ[ x_Symbol ] := True
validvarQ[ x_List ] := Apply[And, Map[VariableQ, x]]
validvarQ[ x_ ] := False


(*  Extension of Fourier transform operators  *)

Unprotect[FT, InvFT]
FT/: TheFunction[ FT[t_, w_] [f_] ] := CTFTransform[f, t, w]
InvFT/: TheFunction[ InvFT[w_, t_] [f_] ] := InvCTFTransform[f, w, t]
Protect[FT, InvFT]


(*  Extension of TheFunction operator  *)

CTFTData/: TheFunction[ CTFTData[x_, var_] ] := x


(*  Magnitude/Phase plot of a CTFT object *)

mppoptions = {	Domain -> Continuous, DomainScale -> Linear,
		MagRangeScale -> Log, PhaseRangeScale -> Degree,
		PlotRange -> All }

CTFTData/: MagPhasePlot[ CTFTData[trans_, vars_], rest___ ] :=
	Apply [	MagPhasePlot, { trans, rest } ~Join~ mppoptions ]


(*  Multidimensional Fourier transform driver  *)

DualFT[ f_, t_, w_, s_, oplist_, dir_ ] :=
	Block [	{cleanUpRule, different, evaluate, evaluationRule, numdims,
		 newexpr, newrules, oldexpr, rewriteRule, simops, state,
		 tempops, trace, uvars},

		Off[ Power::infy ];
		trace = SameQ[ Replace[Dialogue, oplist], All ];
		numdims = Max[1, Length[t]];
		state = initstate[];
		evaluate = If [ TrueQ[$VersionNumber >= 2.0],
				Evaluate, Release ];

		(*  Generate the 1-D FT rules  *)
		newrules = TransformFixUp[ t, w, oplist, dualFT, False,
					   dir, Null, Null ];
		If [ SameQ[dir, InvCTFTransform],
		    uvars = ToList[ DummyVariables[ Length[w], u ] ];
		    newrules = newrules /. ReplaceWith[w, uvars] ];
		DualFTRules = Join[ ToList[newrules], OriginalDualFTRules ];

		(*  Embed the function in the dual ctft object  *)
		If [ trace && SameQ[dir, InvCTFTransform],
		     Print[ "The Duality Theorem will be used to" ];
		     Print[ "find the inverse Fourier transform." ];
		     If [ ListQ[uvars],
			  Print[ "Letting ", 
				 Map[SequenceForm[First[#], " = ", Second[#]]&,
				     Transpose[{uvars, w}]] ],
			  Print[ "Letting ", uvars, " = ", -w, "," ] ] ];

		If [ SameQ[dir, CTFTransform],
		     oldexpr = dualFT[f, t, w, s, fixUp[oplist], state, dir],
		     oldexpr = dualFT[f, t, uvars, s, fixUp[oplist],
		     		      state, InvCTFTransform] / 
			       (2 Pi)^numdims;
		     oldexpr = substitute[ oldexpr, uvars, -w ] ];

		(*  Simplify the function before transformation  *)
		simops = { Dialogue -> trace };
		If [ TrueQ[ $VersionNumber >= 2.0 ],
		     PrependTo[simops, Trig -> False] ];

		If [ TrueQ[ Replace[ Simplify, oplist ] ],
		     tempops = Append[ simops, Variables -> t ];
		     If [ trace, Print[ oldexpr ] ];
		     oldexpr = SPSimplify[ oldexpr, ToCollection[tempops] ];
		     newexpr = oldexpr //. makeRewriteRules[t];
		     If [ trace && SameQ[oldexpr, newexpr],
		          Print[ "which simplifies to" ];
			  Print[ newexpr ] ],
		     newexpr = oldexpr ];

		(*  Primary rule to trigger Fourier transform  *)
		rewriteRule = ( a_dualFT :> Replace[a, DualFTRules] );

		(*  Rule to force functions that hold their arguments to  *)
		(*  evaluate them so that Replace (above) works when held *)
		evaluationRule = ( h_Symbol [ args__ ] :>
				     Map[ evaluate, h[args] ] /;
				     heldFunctionQ[h] );

		(*  Rule to clean up m-D transform calls *)
		cleanUpRule = ( dualFT[ff_, {}, rest__] :> ff );

		(*  Transform the function  *)
		different = True;
		While [ different,
			oldexpr = newexpr;
			newexpr = oldexpr /. rewriteRule;
			newexpr = newexpr //. evaluationRule;
			newexpr = newexpr //. cleanUpRule;
			different = ! SameQ[newexpr, oldexpr];
			If [ different && trace,
			     Print[ "which becomes" ]; Print[ newexpr ] ] ];

		(*  Fix up the transform  *)
		different = True;
		While [ different,
			oldexpr = newexpr;
			newexpr = posttransform[oldexpr];
			different = ! SameQ[newexpr, oldexpr];
			If [ different && trace,
			     Print[ "which becomes" ]; Print[ newexpr ] ] ];

		(*  Check for invalid Laplace transform substitutions  *)
		newexpr = newexpr /.
		  LTransData[e_, rm_, rp_, LVariables[svars_]] :>
		    If [ ROCincludesImagAxis[
			   LTransData[e, rm, rp, LVariables[svars]], oplist],
			 e /. ReplaceWith[svars, I AssociateItem[svars, s, w]],
			 LTransData[e, rm, rp, LVariables[svar]] ];

		newexpr /.
		  LTransData[x_, rest__] :> Message[ Transform::laplace, x ];

		(*  Simplify the resulting transform  *)
		If [ TrueQ[ Replace[ Simplify, oplist ] ],
		     tempops = Append[ simops, Variables -> w ];
		     newexpr = SPSimplify[ newexpr, ToCollection[tempops] ] //.
				 makeRewriteRules[w] ];

		On[ Power::infy ];

		newexpr ]




(*  B E G I N     F O R W A R D     F O U R I E R     T R A N S F O R M  *)


(*  CTFTransform  *)
CTFTransform[ f_ ] :=
	Message[ Transform::novariables, "t (time)", GetVariables[f] ]
CTFTransform[ f_, t_ ] :=
	CTFTransform[ f, t, DummyVariables[Length[t], Global`w] ]
CTFTransform[ f_, t_, w_, options___ ] :=
	Message[ Transform::badvar, "time", CTFTransform, t ] /;
	! validvarQ[t]
CTFTransform[ f_, t_, w_, options___ ] :=
	Message[ Transform::badvar, "frequency", CTFTransform, w ] /;
	! validvarQ[w]
CTFTransform[ f_, t_, w_, options___ ] :=
	Block[ {op, svars, trans, vars},
		vars = Which [ Length[t] > Length[w],
				 DummyVariables[Length[t], Global`w],
			       Length[t] < Length[w],
				 Take[w, Length[t]],
			       True,
				 w ];
		svars = DummyVariables[Length[t], s];

		origVars = t;
		tranVars = vars;
		op = ToList[options] ~Join~ Options[CTFTransform];
		trans = MyFourier[f, t, vars, svars, op];

		If [ InformUserQ[op],
		     Scan[explain, trans, Infinity] ];

		CTFTData[ FourierSimplify[trans], FVariables[vars] ] ]


(*  Multidimensional continuous-time Fourier transform  *)
MyFourier[ f_, t_Symbol, w_, s_, oplist_ ] :=
	DualFT[ f, {t}, {w}, {s}, oplist, CTFTransform ]

MyFourier[ f_, t_List, w_, s_, oplist_ ] :=
	DualFT[ f, t, w, s, oplist, CTFTransform ]


(*  E N D     F O R W A R D     F O U R I E R     T R A N S F O R M  *)




(*  B E G I N     I N V E R S E     F O U R I E R     T R A N S F O R M  *)


(*  InvCTFTransform  *)
InvCTFTransform[ CTFTData[x_, FVariables[w_]] ] := InvCTFTransform[x, w]

InvCTFTransform[ CTFTData[x_, FVariables[w_]], rest__ ] :=
	InvCTFTransform[x, rest]

InvCTFTransform[ x_, w_ ] :=
	InvCTFTransform[ x, w, DummyVariables[Length[w], Global`t] ]

InvCTFTransform[ f_, w_, t_, options___ ] :=
	Message[ Transform::badvar, "frequency", InvCTFTransform, w ] /;
	! validvarQ[w]

InvCTFTransform[ f_, w_, t_, options___ ] :=
	Message[ Transform::badvar, "time", InvCTFTransform, t ] /;
	! validvarQ[t]

InvCTFTransform[ f_, w_, t_, options___ ] :=
	Block[ {op, s, svars, trans, vars},
		op = ToList[options] ~Join~ Options[InvCTFTransform];

		vars = Which [ Length[w] > Length[t],
				 DummyVariables[Length[w], Global`t],
			       Length[w] < Length[t],
				 Take[t, Length[w]],
			       True,
				 t ];

		svars = DummyVariables[Length[w], s];
		origVars = w;
		tranVars = vars;
		trans = MyInverseCTFT[ f, w, vars, svars, op ];

		If [ InformUserQ[op],
		     Scan[explain, trans, Infinity] ];

		FourierSimplify[trans] ]

(*  MultiDInvCTFT  *)
MultiDInvCTFT[ f1_, w_, t_, s_, op_ ]:= 
	Block [	{i, func = f1, length},
		length = Length[t];
		For [ i = 1, i <= length, i++,
		      func = MyInverseCTFT[ func, w[[i]], t[[i]],
					    s[[i]], op ] ];
		func ]

(*  driver for 1D/mD inverse CTFT rules  *)
MyInverseCTFT[ f_, w_Symbol, t_, s_, op_ ] :=
	DualFT[ f, {w}, {t}, {s}, op, InvCTFTransform ]

MyInverseCTFT[ f_, w_List, t_, s_, op_ ] :=
	DualFT[ f, w, t, s, op, InvCTFTransform ]





(*  E N D     I N V E R S E     F O U R I E R     T R A N S F O R M  *)




(*  B E G I N     D U A L     R U L E     B A S E  *)

DualFTRules = {}

OriginalDualFTRules = {

(*  0.  Clean up rules.		*)
(*	a.  Calls to the Integrate routine	*)
dualFT[ If [ a_, b_, c_ ], t_, w_, rest__ ] :>
	If [ a, dualFT[ b, t, w, rest ], dualFT[ c, t, w, rest ] ],

(*      b.  Calls to the Laplace transform rule bases---  must go	*)
(*	    before any other rules to avoid spurious Delta functions;	*)
(*	    InvLaPlace[] returns the head used by its rule base		*)
dualFT[ LTransData[e_, rm_, rp_, LVariables[svars_]], t_, w_, s_, op_, __ ] :>
	If [ ROCincludesImagAxis[LTransData[e, rm, rp, LVariables[svars]], op],
	     e /. ReplaceWith[svars, I AssociateItem[svars, s, w]],
	     LTransData[e, rm, rp, LVariables[svar]] ] /;
	SubsetQ[ToList[svars], s],

dualFT[ InvLaPlace[][e_, svar_Symbol, restL__], t_, w_, s_, rest__ ] :>
Block [	{tvar},
	tvar = AssociateItem[svar, s, t];
	dualFT[ e /. svar -> I t, t, w, s, rest ] ] /;
	MemberQ[s, svar],


(*  I.  Multi-Dimensional Transform Pairs. *)

dualFT[ f_. CPulse[ T_, a_. (x_Symbol^2 + y_Symbol^2) ], t_, w_, rest__ ] :>
Block [ {w1, w2},
	w1 = AssociateItem[ x, t, w ];
	w2 = AssociateItem[ y, t, w ];
	2 Pi (T / a) (BesselJ[ 1, (T / a) Sqrt[w1^2 + w2^2] ] / 
	  Sqrt[w1^2 + w2^2]) resolve[ {x, y}, f, t, w, rest ] ] /;
	SubsetQ[ {x, y}, t ] && MyFreeQ[ f, {x, y} ] && !TrueQ[x == y] &&
	MyFreeQ[ {a, T}, t ],

dualFT[ f_. BesselJ[ 1, a_. Sqrt[x_Symbol^2 + y_Symbol^2] ] / 
	  Sqrt[x_^2 + y_^2], t_, w_, rest__ ] :>
Block [ {w1, w2},
	w1 = AssociateItem[ x, t, w ];
	w2 = AssociateItem[ y, t, w ];
	2 Pi (1 / a) CPulse[ a, w1^2 + w2^2 ] *
	  resolve[ {x, y}, f, t, w, rest ] ] /;
	SubsetQ[ {x, y}, t ] && MyFreeQ[ f, {x, y} ] && !TrueQ[x == y] &&
	  MyFreeQ[ a, t ],

(*  II.  One-Dimensional Transform pairs	*)
(*	 1.  constant functions (JMW)		*)
dualFT[ c_, t_, w_, __ ] :> 
	Block [ {deltaExpr = 1, i, len},
		len = Length[t];
		For [ i = 1, i <= len, i++,
		      deltaExpr *= Delta[ w[[i]] ] ];
		2 Pi c deltaExpr ] /; 
	MyFreeQ[c, t],

(*	 2.  two delta functions	*)
(*	     c delta(t + a) + d delta(t + b) where c = -d and a = -b	*)
(*	     c delta(t - b) - c delta(t + b) <---> -2 I c sin(b w) 	*)
dualFT[ f_. (c_. Delta[x_Symbol + a_] + d_. Delta[x_ + b_]), t_, w_, rest__ ] :>
    -2 I c Sin[ b AssociateItem[ x, t, w ] ] resolve[ x, f, t, w, rest ] /;
    ( a == -b ) && ( c == 1 || c == -1 ) && ( c == -d ) &&
    MemberQ[ t, x ] && FreeQ[f, x] && MyFreeQ[{a,c}, t],

(*	     c delta(t + a) + c delta(t + b) where c = -d and a = -b	*)
(*	     c delta(t - b) + c delta(t + b) <---> 2 c cos(b w) 	*)
dualFT[ f_. (c_. Delta[x_Symbol + a_] + c_. Delta[x_ + b_]), t_, w_, rest__ ] :>
    2 c Cos[ b AssociateItem[ x, t, w ] ] resolve[ x, f, t, w, rest ] /;
    ( a == -b ) &&
    MemberQ[ t, x ] && FreeQ[f, x] && MyFreeQ[{a, c}, t],

(*	 3.  one delta function	*)
dualFT[ f_. (c_. Delta[a_. x_Symbol + b_.]), t_, w_, rest__ ] :>
	(c/Abs[a]) Exp[I (b/a) AssociateItem[ x, t, w ] ] *
	  resolve[ x, f, t, w, rest ] /;
	 MemberQ[ t, x ] && FreeQ[f, x] && MyFreeQ[{a,c}, t],

dualFT[ f_ Delta[a_. x_Symbol + b_.], t_, w_, rest__ ] :>	(* JMW *)
    dualFT[ (f /. (x -> -b/a)) Delta[x + b/a] / Abs[a], t, w, rest ] /;
    MemberQ[ t, x ] && !FreeQ[f, x] && MyFreeQ[{a, b}, t],
    
(*   4.  impulse train (in one dimension only) JMW *)
(* Note: periodicity in multiple dimensions has not yet been implemented *)
dualFT[ f_. Periodic[T_, x_Symbol][g_. Delta[x_ + a_.]], t_, w_, rest__ ] :>
    (1 / Abs[T]) Exp[I a AssociateItem[x, t, w]] *
    	Periodic[2 Pi / T, AssociateItem[x, t, w]][ 
	    dualFT[f g, t, w, rest] ] /;
    MemberQ[ t, x ] && MyFreeQ[{a, g, T}, t],

dualFT[ f_. Summation[n_Symbol, -Infinity, Infinity, 1][
	  y_. Delta[x_Symbol + a_. + n_ T_.]], t_, w_, rest__ ] :>
	(1 / Abs[T]) Exp[I a AssociateItem[x, t, w]] *
	Summation[n, -Infinity, Infinity, 1][
	substitute[ dualFT[f y, t, w, rest], AssociateItem[x, t, w],
	    AssociateItem[x, t, w] + 2 Pi n / T ] ] /;
    MyFreeQ[{a, n, T}, t] && FreeQ[y, n],

(*	 5.  pulse functions		*)
dualFT[ f_. (CStep[x_Symbol + a_] - CStep[x_ + b_]), t_, w_, rest__ ] :>
	2 a Sinc[ a AssociateItem[ x, t, w ] ] resolve[ x, f, t, w, rest ] /;
	(a == -b) && MemberQ[ t, x ] && FreeQ[ f, x ] && MyFreeQ[a, t],

dualFT[ f_. CPulse[T_, x_Symbol + a_.], t_, w_, rest__ ] :>	(* JMW *)
Block [ {w1},
	w1 = AssociateItem[ x, t, w ];
	T Sign[T] Exp[-I w1 T / 2] Exp[I a w1] Sinc[w1 T / 2] *
	  resolve[ x, f, t, w, rest ] ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[{a, T}, t],

(*	6. Two-sided Gaussian functions		*)
(*	   a.  basic transform pair		*)		(* JMW *)
dualFT[ f_. Exp[a_. (b_. x_Symbol + c_.)^2 + d_.], t_, w_, s_, op_, rest___ ] :>
Block [ {w1},
	Assuming[ Negative[a], op ];
	w1 = AssociateItem[ x, t, w ];
	Sqrt[Pi] Exp[(w1^2 / (4 a b^2)) + (I w1 c / b)] /
	  (Sqrt[-a] b) resolve[ x, f Exp[d], t, w, s, op, rest ] ] /;
	mustbenegative[a] && MemberQ[t, x] && FreeQ[{f, d}, x] &&
	  MyFreeQ[{a, b, c}, t],

(*	   b.  alternate form rewrite rule	*)		(* JMW *)
dualFT[ Exp[a_. x_Symbol^2 + b_. x_ + c_.], t_, w_, s_, op_, rest___ ] :>
	Block [ {},
		Assuming[ Negative[a], op ];
		dualFT[ f Exp[ -(Sqrt[-a] x - b / (2 Sqrt[-a]))^2 -
		  (b^2 / (4 a)) + c ], t, w, s, op, rest ] ] /;
	MemberQ[t, x] && FreeQ[{a, b}, x] && (Im[b] == 0) && mustbenegative[a],

(*	   c.  complicated expresion rewrite	*)		(* JMW *)
dualFT[ Exp[ f_. (a_. (b_. x_Symbol + c_.)^2 + d_. (e_. x_ + f_.)^2) ],
    t_, w_, s_, op_, rest___ ] :>
	dualFT[ f Exp[ (a b^2 + d e^2) x^2 + 2 (a b c + d e f) x + 
			a c^2 + d f^2 ], t, w, s, op, rest ] /;
    MemberQ[f, x] && FreeQ[{a, b, c, d, e, f}, x],

(*	7.  One-sided Gaussian with multiply by time/freq *)	(* JMW *)
dualFT[ f_. x_Symbol Exp[ poly_ ] CStep[dir_. x_], t_, w_, rest___ ] :>
	Block [ {beta, gamma, const},
		beta = -Coefficient[ poly, t, 2 ];
		gamma = Sign[dir] (-Coefficient[ poly, t, 1 ] +
			I AssociateItem[ x, t, w]);
		const = Coefficient[ poly, t, 0 ];
		Assuming[ Negative[ Coefficient[ poly, t, 2 ] ], op ];
		(Sign[-dir] Exp[const] / (2 beta)) *
		( ((Sqrt[Pi] gamma / (2 Sqrt[beta])) Exp[gamma^2 / (4 beta)] *
		Erfc[ gamma / (2 Sqrt[beta]) ]) - 1 ) *
		  resolve[ x, f, t, w, rest ] ] /;
	MemberQ[t, x] && FreeQ[f, x] && PolynomialQ[ poly ] &&
	  (Exponent[ poly, x ] == 2) &&
	  MyFreeQ[ { dir,
		     Coefficient[poly, x, 2],
		     Coefficient[poly, x, 1],
		     Coefficient[poly, x, 0] }, t] &&
	  mustbenegative[ Coefficient[ poly, x, 2 ] ],

(*	 7.  cosine x over x in time--- becomes 1 - CPulse[2 a, w + a] 	 *)
(*	     derived by using Cos[x] = ( Exp[I a t] + Exp[-I a t] ) / 2  *)
(*	     CTFT is I Abs[a] ( CStep[-w - Abs[a]] - CStep[w - Abs[a]] ) *)
dualFT[ f_. Cos[a_. x_Symbol] / x_, t_, w_, rest__ ] :>
	I Abs[a] ( 1 - CPulse[2 Abs[a], AssociateItem[x, t, w] + Abs[a]] ) *
	  resolve[ x, f, t, w, rest ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[a, t],

(*	 8.  two-sided cosine		*)
dualFT[ f_. Cos[w0_. x_Symbol + b_.], t_, w_, rest__ ] :>
	Block [ {w1},
		w1 = AssociateItem[ x, t, w ];
		Exp[I w1 b / w0] Pi ( Delta[w1 - w0] + Delta[w1 + w0] ) *
		  resolve[ x, f, t, w, rest ] ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[{b, w0}, t],

(*	 9.  two-sided sine		*)
dualFT[ f_. Sin[w0_. x_Symbol + b_.], t_, w_, rest__ ] :>
	Block [ {w1},
		w1 = AssociateItem[ x, t, w ];
		Exp[I w1 b / w0] Pi / I ( Delta[w1 - w0] - Delta[w1 + w0] ) *
		  resolve[ x, f, t, w, rest ] ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[{b, w0}, t],

(*  10.  two-sided sinc    { Sin[a t] / t = a Sinc[a t] } *)
dualFT[ f_. Sin[a_. x_Symbol] / x_, t_, w_, rest__ ] :>
	Pi CPulse[2 a, AssociateItem[x, t, w] + a] resolve[x, f, t, w, rest] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[a, t],

dualFT[ f_. Sinc[a_. x_Symbol + b_.], t_, w_, rest__ ] :>	(* JMW *)
	Exp[I AssociateItem[x, t, w] b / a] Pi *
	  CPulse[2 a, AssociateItem[x, t, w] + a] / Abs[a] *
	  resolve[ x, f, t, w, rest ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[{a,b}, t],

dualFT[ f_. (Sinc[a_. x_Symbol + b_.])^n_, t_, w_, rest__ ] :>
	Block [ {w1},
		w1 = AssociateItem[x, t, w];
		Exp[I w1 b / a] Pi *
		  SymmetricPulseConvolutions[1/a, a, w1, n] / 2^(n-1) *
		  resolve[ x, f, t, w, rest ] ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[{a,b}, t] &&
	  IntegerQ[n] && (n > 1),

(*	11.  other two-sided forms	*)
dualFT[ f_. Exp[-Abs[a_. x_Symbol]] Sinc[a_. x_], t_, w_, rest__ ] :>
	ArcTan[ 2 a^2 / AssociateItem[x, t, w]^2 ] / a *
	  resolve[ x, f, t, w, rest ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[ a, t ],

dualFT[ f_. Sign[x_Symbol + x0_.], t_, w_, rest__ ] :>		(* JMW *)
	-2 I Exp[I x0 AssociateItem[x, t, w]] / AssociateItem[x, t, w] *
	  resolve[ x, f, t, w, rest ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[x0, t],

dualFT[ f_. / x_Symbol, t_, w_, rest__ ] :>			(* JMW *)
	-I Pi Sign[ AssociateItem[x, t, w] ] resolve[ x, f, t, w, rest ] /;
	MemberQ[t, x] && FreeQ[f, x],

dualFT[ f_. BesselJ[0, a_. x_Symbol], t_, w_, rest__ ] :>
	CPulse[2 a, AssociateItem[x, t, w] + a] /
	  ( Pi Sqrt[a^2 - AssociateItem[x, t, w]^2] ) *
	  resolve[ x, f, t, w, rest ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[a, t],

dualFT[ f_. BesselJ[1, a_. x_Symbol] / x_, t_, w_, rest__ ] :>   (* JMW *)
	2 a CPulse[ 2 a, AssociateItem[x, t, w] + a ] *
	  Sqrt[a^2 - AssociateItem[x, t, w]^2] resolve[ x, f, t, w, rest ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[ a, t ],

(* JMW Unposted *)
dualFT[ f_. (BesselJ[1, a_. x_Symbol] / x_) CStep[dir_. x_], t_, w_, rest__ ] :>
	( -I Sign[dir] AssociateItem[x, t, w] + 
	   CPulse[ 2 a, AssociateItem[x, t, w] + a ] *
	   Sqrt[ a^2 - AssociateItem[x, t, w]^2 ] ) / Abs[a] *
	resolve[ x, f, t, w, rest ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[ a, t ],  

(* JMW Unposted - Not Verified 
dualFT[ f_. (BesselJ[1, a_. x_Symbol^2] / x_) CStep[dir_. x_],
	t_, w_, rest__ ] :>
Block [ {w1},
	w1 = AssociateItem[ x, t, w ];
	(Sign[dir] ((-Sqrt[w1^4/a^2] / 4) + (HypergeometricPFQ[ {-1/2}, 
	  {1/4, 3/4}, -w1^4 / (64 a^2) ] / 2))) - (I (w1^4/a^2)^(1/4) *
	  Gamma[3/4] HypergeometricPFQ[ {-1/4}, {1/2, 5/4}, -w1^4 / 
	  (64 a^2) ] / (2^(3/2) Gamma[5/4])) + (I/3 (w1^4 / a^2)^(3/4) *
	  Gamma[5/4] HypergeometricPFQ[ {1/4}, {3/2, 7/4}, -w1^4 /
	  (64 a^2) ] / (2^(3/2) Gamma[3/4])) *
	  resolve[ x, f, t, w, rest ] ] /;
    MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[a, t],
*)
   
(* JMW *)
dualFT[ f_. BesselJ[1, a_. x_Symbol] CStep[dir_. x_], t_, w_, rest__ ] :>
	Block [ {w1},
		w1 = AssociateItem[ x, t, w ];
		Sqrt[a^2] / a^2 - (Sign[dir] I Sqrt[ w1^2 (a^2-w1^2) / a^2 ] / 
		  (a^2 - w1^2)) *
		  resolve[ x, f, t, w, rest ] ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[a, t],

(* JMW Unposted - Not Verified 
dualFT[ f_. Sqrt[rsq_ - x_Symbol^2] CPulse[T_, x_ + x0_], t_, w_, rest__ ] :>
	Pi BesselJ[ 1, Sqrt[rsq] AssociateItem[x, t, w] ] /
	  AssociateItem[x, t, w] * resolve[ x, f, t, w, rest ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[ {rsq, l, x0}, t ] &&
	  (T^2 == 4 rsq) && (x0^2 == rsq),
*)

(*	12.  one-sided functions--- CStep functions are handled later	*)
dualFT[ f_. CStep[x_Symbol + x0_.], t_, w_, rest__ ] :>
	Block [ {w1},
		w1 = AssociateItem[x, t, w];
		(-I Exp[I x0 w1] / w1 + Pi Delta[w1]) *
		  resolve[x, f, t, w, rest] ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[x0, t],

dualFT[ f_. CStep[-x_Symbol + x0_.], t_, w_, rest__ ] :>
	Block [ {w1},
		w1 = AssociateItem[x, t, w];
		(I Exp[-I x0 w1] / w1 + Pi Delta[w1]) *
		  resolve[x, f, t, w, rest] ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[x0, t],

dualFT[ f_. Unit[a_][x_Symbol + b_.], t_, w_, s_, op_, rest___ ] :>  (* JMW *)
	Block [ {w1},
		Assuming[ Positive[a], op ];
		w1 = AssociateItem[x, t, w];
		Exp[I w1 b] (I w1)^a resolve[x, f, t, w, s, op, rest] ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[{a,b}, t] && mustbepositive[a],

(* JMW *)
dualFT[ f_. Sinc[a_. x_Symbol] CStep[dir_. x_], t_, w_, rest___ ] :> 
	(Pi CPulse[ 2 a, AssociateItem[ x, t, w ] + a ] +
	  Sign[dir] I (Log[ Abs[ (AssociateItem[ x, t, w ] - a) ] ] -
	  Log[ Abs[ (AssociateItem[ x, t, w ] + a) ] ])) / (2 a) *
	  resolve[x, f, t, w, rest] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[{a, dir}, t],


(*  B.  Properties  *)
(*	1.  homogeneity		*)
dualFT[ a_ f_, t_, w_, rest__ ] :>
	ApplyHomogeneityRule[ a f, dualFT, t, w, rest ] /;
	MyFreeQ[a, t],

(*	2.  additivity		*)
dualFT[ a_ + b_, t_, w_, rest__ ] :>
	dualFT[ a, t, w, rest ] + dualFT[ b, t, w, rest ],

dualFT[ Summation[i_, il_, iu_, inc_, options___][f_], t_, w_, rest___ ] :>
	Summation[i, il, iu, inc, options][ dualFT[f, t, w, rest] ] /;
	MyFreeQ[{i, il, iu, inc}, t],

(*      3.  Ratio of two polynomials    *)
(*	    Partial fractions here is a kludge, but the inverse		*)
(*	    Laplace rule base will probably have to do it anyway.       *)
dualFT[ p_, t_, w_, s_, op_, state_, dir_ ] :>
	Block [ {wvar, x},
		x = rationalPolynomialVar[p, t];
		If [ InformUserQ[op],
		     Print[ "( after performing partial fractions on" ];
		     Print[ "  the rational polynomial with respect to ", x ];
		     Print[ "  ", p, " )" ] ];
		wvar = AssociateItem[x, t, w];
		resolve[ x,
			 applyInverseLaplace[ Apart[p, wvar], x, wvar,
					      AssociateItem[x, t, s],
			 		      op, state, dir ],
			 t, w, s, op, state, dir ] ] /;
	rationalPolynomialQ[p, t],

(*	4.  one-sided transform	*)
(*	    a. remove shifting	*)
dualFT[ f_. CStep[x_Symbol + x0_], t_, w_, rest__ ] :>
	Exp[- I AssociateItem[x, t, w] x0] *
	  dualFT[ (f /. x -> (x - x0)) CStep[x], t, w, rest ] /;
	MemberQ[t, x] && MyFreeQ[x0, t],		(* [O&W, 223] *)

(*	    b. take one-sided one-dimensional transform	*)
dualFT[ f_ CStep[a_. x_Symbol], t_, w_, s_, op_, state_, dir_ ] :>
	resolve[ x,
		 applyForwardLaplace[ f CStep[a x], x, AssociateItem[x, t, w],
				      AssociateItem[x, t, s], op, state, dir ],
		 t, w, s, op, state, dir ] /;
	MemberQ[t, x] && (! FreeQ[f, x]) && ( (a == 1) || (a == -1) ) &&
	  ! MemberQ[ GetStateField[state, formultivaluedfield], x ],

(*	5.  derivative 		*)
dualFT[ f_. Derivative[k_][g_][x_Symbol], t_, w_, rest__ ] :>
	I^k AssociateItem[x, t, w]^k dualFT[ f g[x], t, w, rest ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[k, t],

dualFT[ f_. derivative[g_, x_Symbol, k_], t_, w_, rest__ ] :>
	I^k AssociateItem[x, t, w]^k dualFT[ f g, t, w, rest ] /;
	MemberQ[t, x] && FreeQ[f, x] && MyFreeQ[k, t],

(*	6.  integration		*)
(*	    Patched 2/2/94 by RHB to get it to work  *)
dualFT[ f_. Integrate[g_, {tau_Symbol, -Infinity, x_Symbol}],
	  t_, w_, rest__ ] :>
	Block [ {trans, w1, newfunc},
		w1 = AssociateItem[x, t, w];
		newfunc = g /. tau -> x;
		trans = dualFT[newfunc, t, w, rest];
		trans (1/(I w1) + Pi Delta[w1]) resolve[x, f, t, w, rest] ] /;
	MemberQ[t, x] && FreeQ[f, x] && ! FreeQ[g, tau],

(*	7.  modulation JMW	*)
(*	    a. exponential raised to an imaginary power times a function  *)
dualFT[ f_. Exp[Complex[0, w0_] w1_. x_Symbol + restexp_.], t_, w_, rest__ ] :>
	substitute[ dualFT[f Exp[restexp], t, w, rest],
		    AssociateItem[x, t, w],
		    AssociateItem[x, t, w] - w0 w1 ] /;
	MemberQ[t, x] && MyFreeQ[{w0, w1}, t],

(*	    b. special case for Exp[Cos[x] I w] terms.  Should be handled *)
(*	       below--- maybe a bug in the Mathematica pattern matcher.   *)
dualFT[ f_. Exp[ realf_[q_] Complex[0, w0_] w1_. x_Symbol + restexp_. ],
	  t_, w_, rest__ ] :>
	substitute[ dualFT[f Exp[restexp], t, w, rest],
		    AssociateItem[x, t, w],
		    AssociateItem[x, t, w] - realf[q] w0 w1 ] /;
	MemberQ[t, x] && MyFreeQ[{w0, w1}, t] && MemberQ[ {Cos, Sin}, realf ],

(*	    c. special case for Exp[w (Cos[x] I c0 + Sin[y] I c1 )] terms *)
dualFT[ f_. Exp[ (realf_[q_] Complex[0, w0_] + z_.) w1_. x_Symbol + restexp_. ],
	  t_, w_, rest__ ] :>
	substitute[ dualFT[f Exp[z w1 x + restexp], t, w, rest],
		    AssociateItem[x, t, w],
		    AssociateItem[x, t, w] - realf[q] w0 w1 ] /;
	MemberQ[t, x] && MyFreeQ[{w0, w1}, t] && MemberQ[ {Cos, Sin}, realf ],

(*	    d. cosine modulation adapted by JMW				  *)
dualFT[ f_. Cos[w0_. x_Symbol], t_, w_, rest__ ] :>
	Block [ {w1},
		w1 = AssociateItem[x, t, w];
		substitute[ dualFT[f, t, w, rest ] / 2,
			    w1, w1 - w0, Plus, w1 + w0 ] ] /;
	MemberQ[t, x] && MyFreeQ[w0, t],

(*	    e. sine modulation adapted by JMW				  *)
dualFT[ f_. Sin[w0_. x_Symbol], t_, w_, rest__ ] :>
	Block [ {w1},
		w1 = AssociateItem[x, t, w];
		substitute[ -I dualFT[f, t, w, rest] / 2,
			    w1, w1 - w0, Minus, w1 + w0 ] ] /;
	MemberQ[t, x] && MyFreeQ[w0, t],

(*	8.  multiply by time	*)
(*	    a. derivative method for piecewise polynomials (JMW)	*)
dualFT[ f_. x_Symbol^a_. piecewise_[ pargs___ ], t_, w_, s_, op_, rest___ ] :>
	Block[ {dialogue},
		dialogue = SameQ[ Replace[Dialogue, op], All ];
		Assuming[ Positive[a], dialogue ];
		If [ dialogue && ! IntegerQ[a],
		     Print[ "where ", a, " is an integer" ] ];
		dualFT[ f D[ x^a piecewise[pargs], {x, a} ], 
			t, w, s, op, rest ] /
			((I AssociateItem[x, t, w]) ^ a) ] /;
	MemberQ[t, x] && FreeQ[f, x] &&
	  SameQ[piecewise, CPulse] && MyFreeQ[a, t] &&
	  Implies[ NumberQ[a], IntegerQ[a] && a > 0 ],	(* [Bracewell, ch. 7] *)

(*	    b. general multiply-by-time rule  *)
dualFT[ x_Symbol^a_. f_, t_, w_, s_, op_, rest___ ] :>
	Block [	{dialogue},
		dialogue = SameQ[ Replace[Dialogue, op], All ];
		Assuming[ Positive[a], dialogue ];
		If [ dialogue && ! IntegerQ[a],
		     Print[ "where ", a, " is an integer" ] ];
		I^a derivative[ dualFT[f, t, w, s, op, rest],
				AssociateItem[x, t, w], a ] ] /;
	MemberQ[t, x] && MyFreeQ[a, t] &&
	  Implies[ NumberQ[a], IntegerQ[a] && a > 0 ],

(*	9.  divide by time	  *)
dualFT[ f_ / x_Symbol^k_., t_, w_, rest__ ] :>
	-I integrate[ dualFT[f / x^(k-1), t, w, rest],
		      { AssociateItem[x, t, w],
			-Infinity,
			AssociateItem[x, t, w] } ] +
	I Pi Limit[f / x^(k-1), x -> 0] /;
	MemberQ[t, x] && N[ k >= 1 ] && hasValueAtZero[f / x^(k-1), x],


(*  C.  Operators  *)
(* 	1.  convolution (JMW) Unposted (Inv constant)		*)
dualFT[ f_. CConvolve[x_Symbol, options___][g_, restconv__], t_, w_, 
	  s_, oplist_, state_, dir_ ] :>
	Block [ {constant = 1},
		If [ SameQ[ dir, InvCTFTransform ], constant = 2 Pi ];
		constant *
		  Apply[ Times,
			 Map[ dualFT[#1, t, w, s, oplist, state, dir]&,
			      { g, restconv } ] ] *
		  resolve[x, f, t, w, s, oplist, state, dir] ] /;
	MemberQ[t, x] && FreeQ[f, x],

(*	2.a. general 1D periodic function (JMW)	Unposted *)
dualFT[ f_. Periodic[T_, x_Symbol][ e_ ], t_, w_, rest__ ] :>
	(2 Pi / Abs[T]) dualFT[ f e, t, w, rest ] *
	  Periodic[ 2 Pi / T, AssociateItem[x, t, w] ][
	    Delta[ AssociateItem[x, t, w] ] ] /;
	MemberQ[t, x] && FreeQ[{f, T}, x],

(*	2.b. general M-D periodic functions 	(JMW) Unposted 	*)
(* Unfinished ... many problems in pattern below
dualFT[ f_. Periodic[ T_, tlist_List ][ g_ z_. ], t_Symbol, w_, rest___ ] :>
	Block [	{},
		(2 Pi / Abs[T]) dualFT[ f g, t, w, rest ] *
		  Periodic[ 2 Pi / T, w ][ Delta[w] ] ]
*)

(*	3. windowed series of impulses becomes a discrete	*)
(*	   set of function values times impulses (JMW) 		*)
dualFT[ f_ Summation[k_, start_, end_, step_][ Delta[x_Symbol + a_.] ],
	  t_, w_, rest__ ] :>
	dualFT[ Summation[k,start,end,step][ Limit[f, x -> -a] Delta[x+a] ],
		t, w, rest ] /;
	MemberQ[t, x] && FreeQ[a, x] && !FreeQ[f, x],

(*	4.  Shift Operator (JMW)				*)
(*      Modified by RHB to work for 1D signals  (9/17/93)       *)
dualFT[ Shift[ shift_List, vars_List ][ f_ ], t_, w_, rest___ ] :>
	Block [ {i, len, newshift = {}, newvars = {}, trans = 1},
		len = Length[vars];
		For [ i = 1, i <= len, i++,
		      If [ MemberQ[ t, vars[[i]] ],
			   trans *= Exp[ -I shift[[i]] *
					 AssociateItem[ vars[[i]], t, w ] ],
			   newshift = Append[ newshift, shift[[i]] ];
			   newvars = Append[ newvars, vars[[i]] ] ] ];
		If [ EmptyQ[newshift],
		     trans dualFT[ f, t, w, rest ],
		     trans Shift[newshift, newvars][dualFT[f, t, w, rest]] ] ],

dualFT[ Shift[ shift_Symbol, var_Symbol ][ f_ ], t_, w_, rest___ ] :>
	Exp[-I shift AssociateItem[var, t, w]] dualFT[ f, t, w, rest] /;
	MemberQ[t, var],

(*	5.  M-D Scaling of Axis of Transformation (JMW)		*)
(*	    Note - not tested with scale and axes as lists	*)
dualFT[ ScaleAxis[ scale_, axis_ ][ g_ ], t_, w_, rest___ ] :>
Block [ {i, newscale = {}, newaxis = {}, rescale = 1},
	For [ i = 1, i <= Length[axis], i++,
	      If [ MemberQ[ t, axis[[i]] ],
		   newscale = Append[newscale, 1/scale[[i]] ];
		   newaxis = Append[newaxis, AssociateItem[axis[[i]], t, w] ];
		   rescale = rescale / Abs[ scale[[i]] ],
		   newscale = Append[ newscale, scale[[i]] ];
		   newaxis = Append[ newaxis, axis[[i]] ] ] ];
	ScaleAxis[ newscale, newaxis ][ dualFT[ rescale g, t, w, rest ] ] ] /;
	MyFreeQ[ scale, t ],

(*	6. Rotation of Axes. 			(JMW) Unposted	*)
dualFT[ RotateAxes[ angle_, axes_ ][ g_ ], t_, w_, rest___ ] :>
Block [ {i, newaxes={}},
	For [ i = 1, i <= Length[axes], i++,
	      If [ MemberQ[ t, axes[[i]] ],
		   newaxes = Append[newaxes, AssociateItem[axes[[i]], t, w] ],
		   newaxes = Append[ newaxes, axes[[i]] ] ] ];
	RotateAxes[ angle, newaxes ][ dualFT[ g, t, w, rest ] ] ] /;
	MyFreeQ[angle, t],


(*  D.	Strategies		*)
(*	1.  Break Abs[x] terms into positive and negative parts JMW	*)
(*	    the semaphore absfield is set to False if no Abs[timevar]	*)
(*	    forms are found where timevar is one of the time variables	*)
dualFT[ g_, t_, w_, s_, oplist_, state_, dir_ ] :>
	If [ containsAbsQ[g, t],	(* need to split functions BLE *)
	     dualFT[ SPSimplify[g /. (Abs[x] -> x), Variables -> x] CStep[x] +
		     SPSimplify[g /. (Abs[x] -> -x), Variables -> x] CStep[-x],
		     t, w, s, oplist, state, dir ],
	     dualFT[ g, t, w, s, oplist,
		     SetStateField[state, absfield, False], dir ] ] /;
	GetState[state, absfield],

(*	2.  Expand expression	*)
dualFT[ e_, t_, w_, s_, oplist_, state_, rest___ ] :>
	dualFT [ MapAll[Expand, e], t, w, s, oplist,
		 SetStateField[state, expandfield, False], rest ] /;
	GetStateField[state, expandfield],

(*	3.  Use convolution property when multiplying by Sinc functions	*)
(*	    Add factor of 1/(2 Pi) for forward transforms (JMW)		*)
dualFT[ f_ Sinc[a_. x_Symbol + b_.], t_, w_, s_, oplist_, state_, dir_ ] :>
Block [	{constant},
	constant = If [ SameQ[dir, CTFTransform], 2 Pi, 1 ];
	CConvolve[ AssociateItem[ x, t, w ] ][
	  dualFT[f, t, w, s, oplist, state, dir],
	    dualFT[ Sinc[a x + b], {x}, {AssociateItem[ x, t, w ]}, 
		    s, oplist, state, dir ] ] / constant ] /;
	MemberQ[t, x] && !FreeQ[f, x],

(*	4.  function times a window	*)
dualFT[ f_ CPulse[T_, x_Symbol + a_.], t_, w_, s_, op_, state_, dir_ ] :>
Block [ {trans, newstate, w1},
	w1 = AssociateItem[ x, t, w ];
    	trans = Integrate[ TheFunction[f] Exp[-I w1 x], {x, -a, -a+T} ];
	If [ SameQ[Head[trans], Integrate],
	    newstate = SetStateField[ state, integratepulsefield, False ];
	    dualFT[ f CPulse[T, x+a], t, w, s, op, newstate, dir ],
	    Sign[T] resolve[ x, trans, t, w, s, op, state, dir ] ] ] /;
	GetStateField[state, integratepulsefield] && MemberQ[t, x] &&
	  MyFreeQ[{a, T}, t],

(*	5.  Apply the definition if Definition options is enabled.	*)
(*	    For multidimensional transforms, Integration is applied	*)
(*	    to first variable in the variable list.  Then the rule	*)
(*	    base is re-applied--- JMW					*)
dualFT[ f_, t_, w_, s_, op_, state_, dir_ ] :>
Block [ {newstate, trans, soln, result},
	trans = Integrate[ f Exp[- I w[[1]] t[[1]]],
			   {t[[1]], -Infinity, Infinity} ];
	If [ SameQ[Head[trans], Integrate],
	    newstate = SetStateField[state, definitionfield, False];
	    dualFT[f, t, w, s, op, newstate, dir],
	    resolve[ t[[1]], 
	    	TransformDialogue[ Definition, op, Integrate, f, trans ],
		t, w, s, op, state, dir ] ] ] /;
	GetStateField[state, definitionfield] && Replace[Definition, op],

(*	6.  Try a forward or inverse Laplace transform if all else fails  *)
(*	    a. Try forward Laplace if we're doing forward Fourier or if	  *)
(*	       we're doing inverse Fourier and the inverse Laplace choked *)
dualFT[ f_, t_, w_, s_, op_, state_, dir_ ] :>
	applyForwardLaplace[f, t, w, s, op, state, dir ] /;
	GetStateField[state, forlaplacefield] &&
	  (! ( GetStateField[state, invlaplacefield] &&
	      SameQ[dir, InvCTFTransform] )) &&
	  (! MemberQ[ GetStateField[state, formultivaluedfield], t ]),

(*	    b. Try inverse Laplace if we're doing inverse Fourier or if	  *)
(*	       we're doing forward Fourier and the forward Laplace choked *)
dualFT[ f_, w_, t_, s_, op_, state_, dir_ ] :>
	applyInverseLaplace[f, w, t, s, op, state, dir] /;
	GetStateField[state, invlaplacefield] &&
	  (! ( GetStateField[state, forlaplacefield] &&
	      SameQ[dir, CTFTransform] )) &&
	  (! MemberQ[ GetStateField[state, invmultivaluedfield], w ])

}


(*  E N D     P A C K A G E  *)

End[]
EndPackage[]

If [ TrueQ[ $VersionNumber >= 2.0 ],
     On[ General::spell ];
     On[ General::spell1 ] ];


(*  H E L P     I N F O R M A T I O N  *)

Combine[ SPfunctions, { CTFTransform, InvCTFTransform } ]
Protect[ CTFTransform, InvCTFTransform ]


(*  E N D I N G     M E S S A G E  *)

Print[ "The continuous Fourier transform (CTFT) functions are loaded." ]
Null
