(*  :Title:	Mimicking Lisp in Mathematica  *)

(*  :Authors:	Brian L. Evans *)

(*  :Summary:   To provide commonly used Lisp routines in Mathematica.  *)

(*  :Context:	SignalProcessing`Support`Lisp`  *)

(*  :PackageVersion:  SPP 2.9  @(#)Lisp.m	1.1	2/8/94  *)

(*
    :Copyright:	Copyright 1989-1994 by Brian L. Evans
		Georgia Tech Research Corporation
		All Rights Reserved

	Permission to use, copy, and distribute this software and
	its documentation is hereby granted, provided that no profit
	is made, that both the copyright notice and this permission
	notice appear in all copies, that both that copyright notice
	and this permission notice appear in supporting documentation,
	and that the name of the Georgia Tech Research Corporation
	(hereafter "GTRC"), Georgia Tech, or Georgia Institute of
	Technology not be used in advertising or publicity pertaining
	to distribution of the software without specific, written prior
	permission by the GTRC.  Permission to modify this software for
	personal use is also hereby granted provided that the modified
	version of this software is neither copied nor distributed.
	Copying or distributing modified versions of this software,
	making derivative works of this software, and marketing this
	software for profit are prohibited except with specific, written
	permission by the GTRC or by any entity to which the GTRC grants
	this authority.  GTRC makes no representations about the
	suitability of this software for any purpose.  It is provided
	"as is" without express or implied warranty.
 *)

(*  :History:	*)

(*  :Keywords:	*)

(*
    :Source:	     Steele, Guy.  Common Lisp: The Language.  Digital
		Press, 1984.
 *)

(*  :Warning:	*)

(*  :Mathematica Version:  1.2 or 2.0  *)

(*  :Limitation:  *)

(*
    :Discussion:

	  Many Mathematica routines are already equivalent to exisiting Lisp
	routines.  Mathematica uses mixed case in naming functions.  For
	example, "length" becomes "Length".  The naming of predicate (boolean)
	functions are handled differently.  Instead of appending a lower-case
	p at the end of the function name, Mathematica appends a capital Q
	(for query).  For example, "vectorp" becomes "VectorQ" in Mathematica.
	A minus sign is not allowed in the middle of a function name.
	For example, "set-exclusive-or" becomes "SetExclusiveOr" in
	Mathematica (below).

	  Some analogs are not so obvious.  A few are listed below:

	    Lisp	Mathematica
	routine/code	equivalent	Exception/Note

	aref		Part
	append		Join
	concatenate	Join		StringJoin for strings
	cond		Which
	(expt x)	Exp[x]
	(expt b p)	b^p
	let		Module		only available in Mathematica 2
	list-length	Length		Mathematica can't have circular lists
	make-list 	Table		see below
	mapcar		MapThread	only available in Mathematica 2
			Map		if only one list being processed
	member		MemberQ		arguments are switched
	(null x)	SameQ[x, {}]	for lists
			SameQ[x, Null]	for null expressions
	(push i l)	PrependTo[l, i]
	reduce		ListReduce	see below
	remove-if	Select		see below
	set-difference  Complement	more than two sets can be passed
	tree-equal	???		not sure what the equivalent is

	  Making lists with (make-list size &key initialElement) is
	equivalent to Table[initialElement, {size}] in Mathematica.

	  Porting reduce is tricky because its Mathematica equivalent
	Fold always requires an initial value:

	(reduce f list :initial-value x)  <==>  Fold[f, x, list]
	(reduce f list :initial-value x :from-end t)  <==>
		Fold[f, x, Reverse[list]]

	However, it is not straightforward when an initial value is not
	given.  We have implemented all possible combinations via ListReduce.

	  One comment about "nil".  The value of "nil" has two analogs in
	Mathematica.  With respect to boolean logic, "nil" means "False"
	in Mathematica.  With respect to expression, "nil" means "Null"
	in Mathematica.  With respect to lists, "nil" means "{}".

	  The opposite of remove-if is readily available in Mathematica
	as Select:   (remove-if #'atom '(1 2 (3 4)))  would become
	Select[ {1, 2, {3, 4}}, Function[x, ! AtomQ[x]] ].

	  I have written several routines below to encode some of the
	commonly used Lisp routines not available in Mathematica.

*)

(*
  :Functions:	Adjoin
		ButLast
		ComplexQ
		Cons
		Elt
		Every
		Intern
		ListReduce
		Nth
		NthCdr
		RationalQ
		RealQ
		Second
		SetExclusiveOr
		SubsetQ
		Third
 *)


If [ TrueQ[ $VersionNumber >= 2.0 ],
     Unprotect[ ListQ ];
     Clear[ ListQ ];
     Off[ General::spell ];
     Off[ General::spell1 ] ]


(*  B E G I N     P A C K A G E  *)

BeginPackage[ "SignalProcessing`Support`Lisp`",
	      "SignalProcessing`Support`SupCode`" ]


(*  U S A G E     I N F O R M A T I O N  *)

Adjoin::usage =
	"Adjoin[item, list] prepends item to list if it is not already \
	in the list."

ButLast::usage =
	"ButLast[list] and ButLast[list, n] returns list but drops last \
	n elements (by default, n = 1)."

ComplexQ::usage =
	"ComplexQ[z] returns True if z is a complex number, False otherwise."

Cons::usage =
	"Cons[x, list] prepends x to the list. \
	 Cons[x, y] returns {x, y} if y is not a list."

Elt::usage =
	"Elt[list, n] returns the nth element in the list \
	with n=0 indicating the first element."

Every::usage =
	"Every[query, expr] returns True if every element of expr \
	satisfies the test query. \
	 Every[query, expr, levelspec] returns True if every part of the
	expression expr satisfies the test query."

Intern::usage =
	"Intern[string] and Intern[string, package] creates a new \
	symbol with the same name as string in the package given \
	(the current package is used in no package is given)."

ListQ::usage =
	"ListQ[expr] gives True if expr is a List, and False otherwise."

ListReduce::usage =
	"ListReduce[f, list], ListReduce[f, list, fromEndFlag], and \
	ListReduce[f, list, fromEndFlag, initialValue] mimic the Lisp \
	reduce routine. \
	 For example, ListReduce[Plus, {1, 2, 3, 4}, False, 0] gives \
	(((1 + 2) + 3) + 4) or 10. \
	 Mathematica's Fold command is used to implement ListReduce."

Nth::usage =
	"Nth[n, list] returns the nth element in the list \
	with n=0 indicating the first element."

NthCdr::usage =
	"NthCdr[n, list] returns the part of the list from the nth element \
	to the end of the list, with n=0 indicating the first element."

RationalQ::usage =
	"RationalQ[m] returns True if m is a rational number. \
	If m is an integer, then this function also return True, \
	since the set of integers are a subset of rationals."

RealQ::usage =
	"RealQ[z] returns True if z is a floating-point number \
	(has a head of Real), False otherwise.  See RealValuedQ."

Second::usage =
	"Second[list] returns the second element of list."

SetExclusiveOr::usage =
	"SetExclusiveOr[set1, set2, ...] returns a set equal to the union \
	of the sets minus the intersection of the sets."

Some::usage =
	"Some[query, expr] returns True if at least one element of expr \
	satisfies the test query. \
	 Some[query, expr, levelspec] returns True if every part of the
	expression expr satisfies the test query."

SubsetQ::usage =
	"SubsetQ[set1, set2, set3,  ...] returns True if set1 is a subset \
	of set2 and set2 is a subset of set3, etc."

Third::usage =
	"Third[list] returns the third element of list."

(*  E N D     U S A G E     I N F O R M A T I O N  *)


Begin["`Private`"]


If [ ! TrueQ[ $VersionNumber >= 2.0 ],
     Fold[f_, x_, list_] := Last[ Accumulate[f, x, list] ] ]


(*  Adjoin  *)
Adjoin[item_, list_] := If [ MemberQ[list, item], list, Prepend[list, item]]

(*  ButLast  *)
ButLast[list_, n_:1] :=
	If [ n >= Length[list], {}, Drop[list, -n] ] /;
	IntegerQ[n] && n >= 1

(*  ComplexQ  *)
ComplexQ[z_] := NumberQ[z] && SameQ[Head[z], Complex]

(*  Cons  *)
Cons[x_, y_?AtomQ] := {x, y}
Cons[x_, y_] := Join[ {x}, y ]

(*  Elt  *)
Elt[list_, n_] := Part[list, n+1]

(*  Every  *)
Every[ f_, expr_, rest___ ] := Apply[And, Map[f, expr, rest]]

(*  Intern  *)
Intern[ string_String ] := ToExpression[string]
Intern[ string_String, package_String ] :=
	ToExpression[ StringJoin[packages, string] ]

(*  ListQ--  it is an undocumented primitive in Mma 2.0+	*)
(*	     in 2.0, it does not always return True or False	*)
ListQ[object_] := SameQ[Head[object], List]

(*  ListReduce  *)
ListReduce[ f_, list_ ] := ListReduce[ f, list, False ]

ListReduce[ f_, {}, flag_ ] := f[]

ListReduce[ f_, s_Symbol, False ] := s
ListReduce[ f_, {s_}, False ] := s
ListReduce[ f_, list_, False ] :=
	Fold[f, f[ list[[1]], list[[2]] ], Rest[ Rest[list] ] ]

ListReduce[ f_, s_Symbol, True ] := s
ListReduce[ f_, {s_}, True ] := s
ListReduce[ f_, list_, True ] :=
	ListReduce[ f[#2, #1]&, Reverse[list], False ]

ListReduce[ f_, list_, False, x_ ] := Fold[ f, x, list ]
ListReduce[ f_, list_, True, x_ ] := Fold[f[#2, #1]&, x, Reverse[list] ]

(*  Nth  *)
Nth[n_, list_] := nth[n, list, Length[list]]
nth[n_, list_, len_] := ToCollection[{}] /; n >= len
nth[n_, list_, len_] := Part[list, n+1]

(*  NthCdr  *)
NthCdr[n_, list_] := nthcdr[n, list, Length[list]]
nthcdr[n_, list_, len_] := {} /; n >= len
nthcdr[n_, list_, len_] := Take[ list, {n+1, len} ]

(*  RationalQ  *)
RationalQ[z_Integer] := True
RationalQ[z_Rational] := True
RationalQ[z_] := False

(*  RealQ  *)
RealQ[z_] := SameQ[Head[z], Real]

(*  Second  *)
Unprotect[Second]
Second[x_] := x[[2]]
Protect[Second]

(*  SetExclusiveOr  *)
SetExclusiveOr[sets__] := Complement[Union[sets], Intersection[sets]]

(*  Some  *)
Some[ f_, expr_, rest___ ] := Apply[Or, Map[f, expr, rest]]

(*  SubsetQ  *)
SubsetQ[x1_] := True
SubsetQ[x1_, x2_] :=
	Block [ {x1sorted},
		x1sorted = Sort[x1];
		SameQ[x1sorted, Intersection[x1sorted, x2]] ]
SubsetQ[x1_, x2_, x__] := SubsetQ[x1, x2] && SubsetQ[x2, x]

(*  Third  *)
Third[x_] := x[[3]]


(*  E N D     P A C K A G E  *)

End[]
EndPackage[]

If [ TrueQ[ $VersionNumber >= 2.0 ],
     On[ General::spell ];
     On[ General::spell1 ] ]


(*  H E L P     I N F O R M A T I O N  *)

Block [ {newfuns},
	newfuns =
	{ Adjoin,	ButLast,	ComplexQ,	Cons,
	  Elt,		Every,		Intern,		ListQ,
	  ListReduce,	Nth,		NthCdr,		RationalQ,
	  RealQ,	Second,		SetExclusiveOr,	Some,
	  SubsetQ,	Third };
	SPfunctions = Combine[SPfunctions, newfuns];
	Apply[Protect, newfuns];
	Protect[Dialogue] ]

Null
